"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.detectPropTypes = void 0;
const typescript_1 = __importDefault(require("typescript"));
function detectPropTypes(sourceFile, name) {
    for (const statement of sourceFile.statements) {
        if (typescript_1.default.isExpressionStatement(statement) &&
            typescript_1.default.isBinaryExpression(statement.expression) &&
            statement.expression.operatorToken.kind === typescript_1.default.SyntaxKind.EqualsToken &&
            typescript_1.default.isPropertyAccessExpression(statement.expression.left) &&
            typescript_1.default.isIdentifier(statement.expression.left.expression)) {
            const componentName = statement.expression.left.expression.text;
            if (componentName !== name) {
                continue;
            }
            // Look for prop types assignments such as Button.propTypes = {...}.
            if (statement.expression.left.name.text === "propTypes") {
                return statement.expression.right;
            }
        }
        else if (typescript_1.default.isClassDeclaration(statement) && statement.name) {
            const componentName = statement.name.text;
            if (componentName !== name) {
                continue;
            }
            for (const classMember of statement.members) {
                // Look for prop types members.
                if (typescript_1.default.isPropertyDeclaration(classMember) &&
                    classMember.initializer &&
                    classMember.name.getText() === "propTypes") {
                    return classMember.initializer;
                }
            }
        }
    }
    return null;
}
exports.detectPropTypes = detectPropTypes;
