"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.svgrPlugin = void 0;
// @ts-ignore untyped
const core_1 = __importDefault(require("@svgr/core"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const vite_1 = require("vite");
function svgrPlugin({ exportedComponentName, alias, }) {
    return {
        name: "previewjs:svgr",
        async transform(code, id) {
            if (!id.endsWith(".svg")) {
                return;
            }
            // ID can either be an absolute path (when SVG is imported with
            // ./logo.svg) or a relative path (when SVG is imported through an
            // alias like foo/logo.svg).
            let filePath = id;
            for (const [mapFrom, mapTo] of Object.entries(alias)) {
                const matchStart = path_1.default.join(path_1.default.sep, mapFrom);
                if (id.startsWith(matchStart)) {
                    filePath = path_1.default.join(mapTo, path_1.default.relative(matchStart, id));
                }
            }
            let componentCode;
            if (await fs_extra_1.default.pathExists(filePath)) {
                const svg = await fs_extra_1.default.readFile(filePath, "utf8");
                const generatedSvgrCode = await (0, core_1.default)(svg, {}, { componentName: "ReactComponent" });
                componentCode = generatedSvgrCode.replace("export default ReactComponent", `export { ReactComponent as ${exportedComponentName} }`);
            }
            else {
                componentCode = `
import React from 'react';

export const ${exportedComponentName} = () => <div>
  Unable to resolve ${id}
</div>;
        `;
            }
            const res = await (0, vite_1.transformWithEsbuild)((exportedComponentName !== "default" ? code : "") +
                "\n" +
                componentCode, filePath, {
                loader: "jsx",
            });
            return {
                code: res.code,
            };
        },
    };
}
exports.svgrPlugin = svgrPlugin;
